@@ -7,7 +7,6 @@ from datetime import timedelta
 from typing import (
     List,
     NamedTuple,
-    Type,
     Union
 )
 
@@ -38,6 +37,7 @@ class TOUValidator(BaseModel):
 class ConsumptionUnit(str, Enum):
     kWh = 'kWh'
     kVA = 'kVA'
+    kW = 'kW'
     day = 'day'
     month = 'month'
 
@@ -170,7 +170,6 @@ class DemandCharge(Charge):
 
     rate: float
     frequency_applied: str
-    tou: Union[TOUValidator, None] = None
 
     # def cross_validate(self):
     #     if not any([self.rate, self.tou]):
@@ -180,6 +179,38 @@ class DemandCharge(Charge):
     #             f'a tou or rate field'
     #         )
 
+    def calculate_charge(
+            self,
+            meter_ts: pd.DataFrame,
+            detailed_bill=False
+    ) -> pd.DataFrame:
+        periods = period_schema[self.frequency_applied]
+        grouped_by_max = get_period_statistic(
+            meter_ts,
+            self.calculate_on,
+            'max',
+            periods,
+        )
+        grouped_by_max.set_index('period_start', inplace=True)
+        bill = pd.concat([meter_ts, grouped_by_max], axis=1)
+        bill[f'rate ({self.rate_unit})'] = self.rate
+        bill['period_peak'] = bill['max']
+        bill[self.name] = self.adjustment_factor * bill['period_peak'] * bill[f'rate ({self.rate_unit})']
+
+        if detailed_bill:
+            return bill
+        else:
+            return bill[self.name]
+
+
+@validate_arguments
+@dataclass
+class TOUDemandCharge(Charge):
+    rate: float
+    frequency_applied: str
+    tou: Union[TOUValidator, None] = None
+
+    #TODO: Fix: This calc works but only for unique peak demand values
     def calculate_charge(
             self,
             meter_ts: pd.DataFrame,
@@ -213,8 +244,8 @@ class DemandCharge(Charge):
             )
 
         bill[f'rate ({self.rate_unit})'] = self.rate
-        max_idx = meter_ts.groupby(bins)[self.calculate_on].transform(max) == meter_ts[self.calculate_on]
-        bill['peaks'] = meter_ts[self.calculate_on][max_idx]
+        max_mask = meter_ts.groupby(bins)[self.calculate_on].transform(max) == meter_ts[self.calculate_on]
+        bill['peaks'] = meter_ts[self.calculate_on][max_mask]
         bill[self.name] = self.adjustment_factor * bill['peaks'] * bill[f'rate ({self.rate_unit})']
 
         if detailed_bill:
@@ -223,6 +254,8 @@ class DemandCharge(Charge):
             return bill[self.name]
 
 
+
+
 @validate_arguments
 @dataclass
 class CapacityCharge(Charge):
@@ -297,15 +330,25 @@ class TariffRegime:
         if tariff_json:
             self.name = tariff_json['name']
             charges = tariff_json['charges']
-            self.charges = []
+            self.charges_dict = {}
             errors_dict = {}
             for charge in charges:
                 try:
                     # Instantiate charge with string of type and dict of attrs
                     validated_charge = globals()[charge['charge_type']](**charge)
-                    self.charges.append(validated_charge)
+                    self.charges_dict[validated_charge.name] = validated_charge
                 except ValidationError as e:
                     errors_dict[charge['name']] = e.errors()
             print(errors_dict)
         if tariff_list:
-            self.charges = tariff_list
+            self.charges_dict = {x.name: x for x in tariff_list}
+
+    @property
+    def charges(self):
+        return list(self.charges_dict.values())
+
+    def delete_charge(self, charge_name: str):
+        del self.charges_dict[charge_name]
+
+    def add_charge(self, charge: Charge):
+        self.charges_dict[charge.name] = charge
